@@ -1845,11 +1845,23 @@
 ;;; ============================== <dirent.h> ================================
 (c-lines "#include <dirent.h>~%")
 
-;;; ----------------------------- <direntry.h> -------------------------------
+;;; ----------------------------- <bits/dirent.h> ---------------------------
+;; d_type is only in dirent64, not in dirent in <linux/dirent.h>,
+;; but it appears to BE required, and does appear in <bits/dirent.h>
+
+(c-lines "#ifndef __USE_FILE_OFFSET64
+typedef __ino_t clisp_dirent_ino_t;
+typedef __off_t clisp_dirent_off_t;
+#else
+typedef __ino64_t clisp_dirent_ino_t;
+typedef __off64_t clisp_dirent_off_t;
+#endif~%")
+(def-c-type clisp_dirent_ino_t)
+(def-c-type clisp_dirent_off_t)
 
 (def-c-struct dirent
-  (d_ino ino_t)
-  (d_off off_t)
+  (d_ino clisp_dirent_ino_t)
+  (d_off clisp_dirent_off_t)
   (d_reclen ushort)
   (d_type uchar)
   (d_name (c-array-max character #.(cl:+ NAME_MAX 1))))
@@ -2390,11 +2402,11 @@ and SIG_IGN do anyway, by other means ... They are just sugar, really.
 ;;; --------------------------------- <bits/sigset.h> -----------------------
 (eval-when (load compile eval)
   (defconstant SIGSET_NWORDS
-    ;; #.(cl:/ 1024 #.(cl:* 8 (ffi:sizeof 'ffi:uint)))
-    32))
+    ;; 32 on 32-bit platforms, 16 on 64-bit platforms
+    #.(cl:/ 1024 #.(cl:* 8 (ffi:sizeof 'ffi:ulong)))))
 
 (def-c-struct sigset_t
-  (val (c-array-max uint #.SIGSET_NWORDS)))
+  (val (c-array-max ulong #.SIGSET_NWORDS)))
 ;;; --------------------------------- <bits/sigaction.h> ---------------------
 
 (def-c-type sighandler_t (c-function (:arguments (sig int)) ;from signal.h
@@ -2403,8 +2415,19 @@ and SIG_IGN do anyway, by other means ... They are just sugar, really.
 (def-c-struct sigaction
   (sa_handler sighandler_t)
   (sa_mask sigset_t)
-  (sa_flags uint32) ; actually int but otherwise CLISP can't coerce SA_RESETHAND?!
-  (sa_restorer (c-function (:arguments) (:return-type nil))))
+  (sa_flags uint32) ; actually int but otherwise CLISP can't coerce SA_RESETHAND
+  ;; The sa_restorer element is obsolete and should not be used.
+  ;; POSIX does not specify a sa_restorer element.
+  (sa_restorer c-pointer))      ; (c-function (:arguments) (:return-type nil))
+
+#+(or)(progn ;; DEBUG
+(def-c-const sigaction-size (:name "sizeof(struct sigaction)") (:guard nil))
+(def-c-const sigset-size (:name "sizeof(sigset_t)") (:guard nil))
+(def-c-const _SIGSET_NWORDS)
+(assert (= (sizeof 'sigaction) sigaction-size))
+(assert (= (sizeof 'sigset_t) sigset-size))
+(assert (= SIGSET_NWORDS _SIGSET_NWORDS))
+)
 
 (defconstant SA_NOCLDSTOP  1)   ; Don't send SIGCHLD when children stop.
 (defconstant SA_NOCLDWAIT  2)   ; Don't create zombie on child death.
