@@ -366,9 +366,6 @@ set *HTTP-PROXY*, and return it; otherwise just return *HTTP-PROXY*."
 (setf (documentation (find-package "CHARSET") 'sys::impnotes) "charset")
 (setf (documentation (find-package "CLOS") 'sys::impnotes) "classes")
 (setf (documentation (find-package "EXT") 'sys::impnotes) "ext-pac")
-#+AFFI
-(when (find-symbol "%LIBCALL" "SYSTEM")
-  (setf (documentation (find-package "AFFI") 'sys::impnotes) "affi"))
 
 (defun clhs (symbol &key (browser *browser*) (out *standard-output*))
   "Dump the CLHS doc for the symbol."
