@@ -1,7 +1,7 @@
 /*
  * Finding the full path of the executable.
  * Bruno Haible 20.12.1994
- * Sam Steingold 2004-2005
+ * Sam Steingold 2004-2006
  */
 
 /* This assumes that the executable is not removed or renamed while
@@ -47,6 +47,10 @@ static int maybe_executable (const char * filename) {
 }
 #endif
 
+/* return the executable name */
+char *get_executable_name (void);
+char *get_executable_name (void) { return executable_name; }
+
 /* find_executable(program_name)
  is to be called immediately after the program starts,
  with program_name = argv[0],
@@ -132,6 +136,16 @@ int find_executable (const char * program_name) {
     if (realpath(program_name,executable_name) == NULL) {
       free(executable_name); goto notfound;
     }
+#if defined(UNIX_CYGWIN32)
+    { /* cygwin does not append ".exe" on its own */
+      int len = strlen(executable_name);
+      if (!(len > 4 && (executable_name[len-4] == '.') &&
+            (executable_name[len-1] == 'e' || executable_name[len-1] == 'E') &&
+            (executable_name[len-2] == 'x' || executable_name[len-2] == 'X') &&
+            (executable_name[len-3] == 'e' || executable_name[len-3] == 'E')))
+        strcat(executable_name,".exe");
+    }
+#endif
     return 0;
   }
   errno = ENOENT;
