@@ -83,12 +83,20 @@
   "The final part of the prompt")
 (defun prompt-finish () (prompt-to-string *prompt-finish*))
 
+;; http://sourceforge.net/tracker/index.php?func=detail&aid=1865636&group_id=1355&atid=101355
+;; [ 1865636 ] infinite recursive error reporting
+(defun safe-wr-st (string &optional (stream *standard-output*))
+  (handler-case (write-string string stream)
+    (system::charset-type-error ()
+      (format stream "Unprintable message: change ~S or ~S" '*current-language*
+              '*terminal-encoding*))))
+
 ;; Help-function:
 (defvar *key-bindings* nil)     ; list of key-bindings and help strings
 (defun help ()
   (dolist (s *key-bindings*)
     (when (stringp s)
-      (write-string s #|*debug-io*|#))))
+      (safe-wr-st s #|*debug-io*|#))))
 
 (defvar *saved-debug-package* *common-lisp-user-package*)
 (defvar *saved-debug-readtable* (copy-readtable nil))
@@ -102,27 +110,27 @@
 ;; Components of the Break-Loop:
 (defvar *debug-frame*)
 (defvar *debug-mode* 4)
-; lower bound for frame-down/frame-down-1
-(defvar *frame-limit1* nil)
-; upper bound for frame-up and frame-up-1
-(defvar *frame-limit2* nil)
+; lower bound for frame-down
+(defvar *frame-limit-down* nil)
+; upper bound for frame-up
+(defvar *frame-limit-up* nil)
 
-(defun frame-limit1 (frames-to-skip)
+(defun frame-limit-down (frames-to-skip)
   (let ((frame (the-frame)))
-    (let ((*frame-limit1* nil)
-          (*frame-limit2* nil))
-      (dotimes (i frames-to-skip) (setq frame (frame-up-1 frame 1))))
+    (let ((*frame-limit-down* nil)
+          (*frame-limit-up* nil))
+      (dotimes (i frames-to-skip) (setq frame (frame-up 1 frame 1))))
     frame))
 
-(defun frame-limit2 ()
+(defun frame-limit-up ()
   (let ((frame (the-frame)))
-    (let ((*frame-limit1* nil)
-          (*frame-limit2* nil))
+    (let ((*frame-limit-down* nil)
+          (*frame-limit-up* nil))
       (loop
-       (let ((nextframe (frame-up-1 frame 1)))
+       (let ((nextframe (frame-up 1 frame 1)))
          (when (or (eq nextframe frame) (driver-frame-p nextframe)) (return))
          (setq frame nextframe)))
-      (dotimes (i 2) (setq frame (frame-down-1 frame 1))))
+      (dotimes (i 2) (setq frame (frame-down 1 frame 1))))
     frame))
 
 (defun debug-help () (help) (throw 'debug 'continue))
@@ -140,29 +148,31 @@
 
 (defun debug-up ()
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-up-1 *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-up 1 *debug-frame* *debug-mode*)))
   (throw 'debug 'continue))
 
 (defun debug-top ()
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-up *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-up t *debug-frame* *debug-mode*)))
   (throw 'debug 'continue))
 
 (defun debug-down ()
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-down-1 *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-down 1 *debug-frame*
+                                                  *debug-mode*)))
   (throw 'debug 'continue))
 
 (defun debug-bottom ()
   (describe-frame *standard-output*
-                  (setq *debug-frame* (frame-down *debug-frame* *debug-mode*)))
+                  (setq *debug-frame* (frame-down t *debug-frame*
+                                                  *debug-mode*)))
   (throw 'debug 'continue))
 
 (defun get-frame-limit ()
   (let (number)
     (loop
      (fresh-line *debug-io*)
-     (write-string (TEXT "Enter the limit for max. frames to print or ':all' for all: ") *debug-io*)
+     (safe-wr-st (TEXT "Enter the limit for max. frames to print or ':all' for all: ") *debug-io*)
      (setq number (read-from-string (read-line *debug-io* nil nil)))
      (if (or (integerp number) (eq number :all))
        (return)
@@ -179,16 +189,17 @@
   (setq *debug-print-frame-limit* (get-frame-limit))
   (throw 'debug 'continue))
 
+(defun frame-up-down (limit mode) (frame-down 1 (frame-up 1 limit mode) mode))
+
 (defun print-backtrace (&key ((:out *standard-output*) *standard-output*)
                         (mode *debug-mode*) (limit *debug-print-frame-limit*))
   ;; SHOW-STACK prints its output to *STANDARD-OUTPUT*, so we bind that
   (let ((frame-count
          (show-stack
           mode limit
-          (frame-down-1 (frame-up-1 (or *frame-limit1* (frame-limit1 13)) mode)
-                        mode))))
+          (frame-up-down (or *frame-limit-down* (frame-limit-down 13)) mode))))
     (fresh-line *standard-output*)
-    (format *standard-output* (TEXT "Printed ~D frames") frame-count)
+    (format *standard-output* (TEXT "Printed ~D frame~:P") frame-count)
     (elastic-newline *standard-output*)))
 
 ;;; debug-backtrace with-optional 'print-limit'
@@ -232,7 +243,7 @@
 ;;; print it
 (defun print-error (condition)
   (fresh-line *debug-io*)
-  (write-string (TEXT "The last error:") *debug-io*)
+  (safe-wr-st (TEXT "The last error:") *debug-io*)
   (pretty-print-condition condition *debug-io* :text-indent 3)
   (elastic-newline *debug-io*))
 
@@ -408,24 +419,27 @@ Continue       :c       switch off single step mode, continue evaluation
          commandsr
          (wrap-user-commands *user-commands*)))
 
+;; establish the DEBUG catcher and the ABORT restart
+(defmacro with-abort-restart ((&key report) &body body)
+  `(catch 'debug
+     (with-restarts ((ABORT () :report (lambda (s) (write-string ,report s))
+                       (throw 'debug 'continue)))
+       ,@body)))
+
 ;; Main-Loop with additional help-command
 (defun main-loop ()
   (setq *break-count* 0)
   (driver                 ; build driver-frame; do #'lambda "infinitely"
    #'(lambda ()
-       (catch 'debug            ; catch the (throw 'debug ...)
+       (with-abort-restart (:report (TEXT "Abort main loop"))
          ;; ANSI CL wants an ABORT restart to be available.
-         (with-restarts ((ABORT () (throw 'debug 'continue)))
-           (when (read-eval-print   ; read-eval-print INPUT-line
-                   (string-concat (prompt-start) (prompt-body) (prompt-finish))
-                   (commands0))
-             ;; T -> #<EOF>
-             ;; NIL -> form is already evaluated
-             ;;        result has been printed
-             (if (interactive-stream-p *standard-input*)
-               (exit)           ; user typed EOF
-               (progn (setq *command-index* 0) ; reset *command-index*
-                      (return-from main-loop))))))))) ; and proceed
+         (when (read-eval-print   ; read-eval-print INPUT-line
+                (string-concat (prompt-start) (prompt-body) (prompt-finish))
+                (commands0))
+           ;; T -> #<EOF>
+           ;; NIL -> form is already evaluated
+           ;;        result has been printed
+           (exit))))))
 
 (setq *driver* #'main-loop)
 
@@ -463,7 +477,7 @@ Continue       :c       switch off single step mode, continue evaluation
     (if may-continue
       (progn
         (write-string "** - " *error-output*)
-        (write-string (TEXT "Continuable Error") *error-output*)
+        (safe-wr-st (TEXT "Continuable Error") *error-output*)
         (terpri *error-output*))
       (write-string "*** - " *error-output*))
 
@@ -472,7 +486,7 @@ Continue       :c       switch off single step mode, continue evaluation
       (if (> *recursive-error-count* 3)
         (progn
           (setq *recursive-error-count* 0)
-          (write-string (TEXT "Unprintable error message.")
+          (write-string (TEXT "Unprintable error message")
                         *error-output*))
         (pretty-print-condition condition *error-output*
                                 :text-indent (if may-continue 5 6))))
@@ -484,14 +498,14 @@ Continue       :c       switch off single step mode, continue evaluation
       (if continuable
         (when interactive-p
           (fresh-line *debug-io*)
-          (write-string (TEXT "You can continue (by typing 'continue').")
-                        *debug-io*)
+          (safe-wr-st (TEXT "You can continue (by typing 'continue').")
+                      *debug-io*)
           (elastic-newline *debug-io*))
         (progn
           (fresh-line *debug-io*)
           (when interactive-p
-            (write-string (TEXT "If you continue (by typing 'continue'): ")
-                          *debug-io*))
+            (safe-wr-st (TEXT "If you continue (by typing 'continue'): ")
+                        *debug-io*))
           (princ may-continue *debug-io*)
           (elastic-newline *debug-io*)))))
 
@@ -503,7 +517,7 @@ Continue       :c       switch off single step mode, continue evaluation
       (when restarts
         (when interactive-p
           (fresh-line *debug-io*)
-          (write-string restarts-help *debug-io*)
+          (safe-wr-st restarts-help *debug-io*)
           (elastic-newline *debug-io*))
         (let ((counter 0))
           (dolist (restart restarts)
@@ -517,7 +531,7 @@ Continue       :c       switch off single step mode, continue evaluation
               ;; display the restarts:
               (when interactive-p
                 (fresh-line *debug-io*)
-                (write-string helpstring *debug-io*)
+                (safe-wr-st helpstring *debug-io*)
                 (elastic-newline *debug-io*))
               (push (string-concat (string #\Newline) helpstring) commandsr)
               ;; put it into the commandsr list.
@@ -537,29 +551,25 @@ Continue       :c       switch off single step mode, continue evaluation
            (*standard-output* stream)
            (prompt (string-concat (prompt-start) (prompt-break)
                                   (prompt-body) (prompt-finish)))
-           (*frame-limit1* (frame-limit1 13))
-           (*frame-limit2* (frame-limit2))
+           (*frame-limit-down* (frame-limit-down 13))
+           (*frame-limit-up* (frame-limit-up))
            (*debug-mode* *debug-mode*)
-           (*debug-frame*
-            (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*)
-                          *debug-mode*))
+           (*debug-frame* (frame-up-down *frame-limit-down* *debug-mode*))
            (commands-list (commands may-continue commandsr)))
       (driver
        ;; build driver frame and repeat #'lambda (infinitely; ...)
        #'(lambda ()
-           (case (catch 'debug    ; catch (throw 'debug ...) and analyse
+           (case (with-abort-restart (:report (TEXT "Abort debug loop"))
                    ;; ANSI CL wants an ABORT restart to be available.
-                   (with-restarts ((ABORT () (throw 'debug 'continue)))
-                     ;; build environment *debug-frame*
-                     ;; which is valid/equal for/to *debug-frame*
-                     (same-env-as *debug-frame*
-                       #'(lambda ()
-                           (if (read-eval-print ; read-eval-print INPUT-line
-                                 prompt commands-list)
-                             ;; T -> #<EOF>
-                             ;; NIL -> form is already evaluated;
-                             ;;        result has been printed
-                             (throw 'debug (if may-continue 'quit 'unwind)))))))
+                   ;; build environment *debug-frame*
+                   ;; which is valid/equal for/to *debug-frame*
+                   (same-env-as *debug-frame*
+                     #'(lambda ()
+                         (if (read-eval-print prompt commands-list)
+                           ;; T -> #<EOF>
+                           ;; NIL -> form is already evaluated;
+                           ;;        result has been printed
+                           (throw 'debug (if may-continue 'quit 'unwind))))))
              ((print-error) (print-error condition))
              ((inspect-error) (inspect condition))
              ((unwind) (go unwind))
@@ -596,15 +606,15 @@ Continue       :c       switch off single step mode, continue evaluation
 (defun step-values (values)
   (let ((*standard-output* *debug-io*))
     (fresh-line #|*debug-io*|#)
-    (write-string (TEXT "step ") #|*debug-io*|#)
+    (safe-wr-st (TEXT "step ") #|*debug-io*|#)
     (write *step-level* #|:stream *debug-io*|#)
     (write-string " ==> " #|*debug-io*|#)
     (case (length values)
-      (0 (write-string (TEXT "no values") #|*debug-io*|#))
-      (1 (write-string (TEXT "value: ") #|*debug-io*|#)
+      (0 (safe-wr-st (TEXT "no values") #|*debug-io*|#))
+      (1 (safe-wr-st (TEXT "value: ") #|*debug-io*|#)
          (write (car values) #|:stream *debug-io*|#))
       (t (write (length values) #|:stream *debug-io*|#)
-         (write-string (TEXT " values: ") #|*debug-io*|#)
+         (safe-wr-st (TEXT " values: ") #|*debug-io*|#)
          (do ((L values))
              ((endp L))
            (write (pop L) #|:stream *debug-io*|#)
@@ -631,15 +641,13 @@ Continue       :c       switch off single step mode, continue evaluation
              (*standard-output* stream)
              (prompt (string-concat (prompt-start) (prompt-step)
                                     (prompt-body) (prompt-finish)))
-             (*frame-limit1* (frame-limit1 11))
-             (*frame-limit2* (frame-limit2))
+             (*frame-limit-down* (frame-limit-down 11))
+             (*frame-limit-up* (frame-limit-up))
              (*debug-mode* *debug-mode*)
-             (*debug-frame*
-              (frame-down-1 (frame-up-1 *frame-limit1* *debug-mode*)
-                            *debug-mode*))
+             (*debug-frame* (frame-up-down *frame-limit-down* *debug-mode*))
              (commands-list (commands nil (commands4))))
         (fresh-line #|*debug-io*|#)
-        (write-string (TEXT "step ") #|*debug-io*|#)
+        (safe-wr-st (TEXT "step ") #|*debug-io*|#)
         (write *step-level* #|:stream *debug-io*|#)
         (write-string " --> " #|*debug-io*|#)
         (write form #|:stream *debug-io*|# :length 4 :level 3)
@@ -651,22 +659,19 @@ Continue       :c       switch off single step mode, continue evaluation
                   ;;  build driver frame and repeat #'lambda (infinitely ...)
                   #'(lambda ()
                       ;; catch the (throw 'debug ...) and analyse
-                      (case
-                          (catch 'debug
-                            ;; ANSI CL wants an ABORT restart to be available.
-                            (with-restarts ((ABORT () (throw 'debug 'continue)))
+                      (case (with-abort-restart (:report (TEXT "Abort stepper"))
+                              ;; ANSI CL wants an ABORT restart to be available.
                               (same-env-as *debug-frame*
                                 ;; build environment *debug-frame* that
                                 ;; is valid/equal for/to *debug-frame*
                                 #'(lambda ()
-                                    (if (read-eval-print ; get/read INPUT-line
-                                          prompt commands-list)
+                                    (if (read-eval-print prompt commands-list)
                                       ;; T -> #<EOF>
                                       (go continue)
                                       ;; NIL -> form is already evaluated;
                                       ;;        result has been printed
                                       #|(throw 'debug 'continue)|#
-                                      )))))
+                                      ))))
                         ((unwind) (go unwind))
                         ((abort-to-top) (go abort-to-top))
                         (t ))))) ; other cases, especially continue
